@@ -5,9 +5,9 @@
 #'@param hillshade A three-dimensional RGB array or 2D matrix of shadow intensities. 
 #'@param overlay A three or four dimensional RGB array, where the 4th dimension represents the alpha (transparency) channel. 
 #'If the array is 3D, `alphacolor` should also be passed to indicate transparent regions.
-#'@param alphacolor Default `NULL`. If `overlay` is a 3-layer array, this argument tells which color is interpretted as completely transparent.
 #'@param alphalayer Default `1`. Defines minimum tranparaency of layer. If transparency already exists in `overlay`, the way `add_overlay` combines 
 #'the two is determined in argument `alphamethod`.
+#'@param alphacolor Default `NULL`. If `overlay` is a 3-layer array, this argument tells which color is interpretted as completely transparent.
 #'@param alphamethod Default `max`. Method for dealing with pre-existing transparency with `layeralpha`. 
 #'If `max`, converts all alpha levels higher than `layeralpha` to the value set in `layeralpha`. Otherwise,
 #'this just sets all transparency to `layeralpha`.
@@ -16,10 +16,8 @@
 #'@return Hillshade with overlay.
 #'@export
 #'@examples
-#'#Only run these examples if the `magick` package is installed.
-#'if ("magick" %in% rownames(utils::installed.packages())) {
 #'#Combining base R plotting with rayshader's spherical color mapping and raytracing:
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'   sphere_shade() %>%
 #'   add_overlay(height_shade(montereybay),alphalayer = 0.6)  %>%
@@ -27,7 +25,7 @@
 #'   plot_map()
 #'}
 #'
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'#Add contours with `generate_contour_overlay()`
 #'montereybay %>%
 #'   height_shade() %>%
@@ -35,9 +33,8 @@
 #'   add_shadow(ray_shade(montereybay,zscale=50)) %>%
 #'   plot_map()
 #'}
-#'}
-add_overlay = function(hillshade, overlay, alphacolor=NULL, 
-                       alphamethod = "max", alphalayer = 1, rescale_original = FALSE) {
+add_overlay = function(hillshade, overlay, alphalayer = 1, alphacolor=NULL, 
+                       alphamethod = "max", rescale_original = FALSE) {
   if(any(alphalayer > 1 || alphalayer < 0)) {
     stop("Argument `alphalayer` must not be less than 0 or more than 1")
   }
@@ -75,7 +72,9 @@ add_overlay = function(hillshade, overlay, alphacolor=NULL,
   if(!is.null(alphacolor)) {
     colorvals = col2rgb(alphacolor)/255
     alphalayer1 = overlay[,,1] == colorvals[1] & overlay[,,2] == colorvals[2] & overlay[,,3] == colorvals[3]
-    overlay[alphalayer1,4] = 0
+    temp_over = overlay[,,4]
+    temp_over[alphalayer1] = 0
+    overlay[,,4] = temp_over
   }
   if(length(dim(hillshade)) == 2) {
     rayimage::add_image_overlay(fliplr(t(hillshade)),overlay,alpha=alphalayer, rescale_original = rescale_original)
