@@ -29,9 +29,7 @@
 #'@return Semi-transparent overlay with a compass.
 #'@export
 #'@examples
-#'#Only run these examples if the `magick` package is installed.
-#'if ("magick" %in% rownames(utils::installed.packages())) {
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'#Create the water palette
 #'water_palette = colorRampPalette(c("darkblue", "dodgerblue", "lightblue"))(200)
 #'bathy_hs = height_shade(montereybay, texture = water_palette)
@@ -49,35 +47,42 @@
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'
+#'if(rayshader:::run_documentation()) {
 #'#Change the position to be over the water
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Change the text color for visibility
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, text_color="white")) %>% 
 #'  plot_map()
-#'  
+#'} 
+#'if(rayshader:::run_documentation()) {
 #'#Alternatively, add a halo color to improve contrast
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, y=0.15,
 #'              halo_color="white", halo_expand = 1)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Alternatively, add a halo color to improve contrast
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, y=0.15,
 #'              halo_color="white", halo_expand = 1)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Change the color scheme
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, y=0.15,
 #'              halo_color="white", halo_expand = 1, color1 = "purple", color2 = "red")) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Remove the inner border
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, y=0.15,
@@ -85,7 +90,8 @@
 #'              halo_color="white", halo_expand = 1, 
 #'              color1 = "darkolivegreen4", color2 = "burlywood3")) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Change the size of the compass and text
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.75, y=0.75,
@@ -98,7 +104,8 @@
 #'              halo_color="white", halo_expand = 1, 
 #'              size=0.075/2, text_size = 0.75)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Change the bearing of the compass
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.85, y=0.85,
@@ -111,7 +118,8 @@
 #'              halo_color="white", halo_expand = 1, bearing=-45,
 #'              size=0.075)) %>% 
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Create a drop shadow effect
 #'base_map %>% 
 #'  add_overlay(generate_compass_overlay(heightmap = montereybay, x = 0.15, y=0.15,
@@ -119,7 +127,6 @@
 #'              halo_color="black", halo_expand = 1, halo_offset = c(0.003,-0.003))) %>% 
 #'  plot_map()
 #'}
-#'}
 generate_compass_overlay = function(x=0.85, y=0.15, 
                                     size=0.075, text_size=1, bearing=0,
                                     heightmap = NULL, width=NA, height=NA, 
@@ -169,7 +176,7 @@ generate_compass_overlay = function(x=0.85, y=0.15,
   grDevices::dev.off() #resets par
   overlay_temp = png::readPNG(tempoverlay)
   if(!is.na(halo_color)) {
-    if(!("rayimage" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("rayimage", quiet = TRUE)) > 0)) {
       stop("{rayimage} package required for `halo_color`")
     }
     tempoverlay = tempfile(fileext = ".png")
