@@ -1,10 +1,13 @@
 #'@title Generate Label Overlay
 #'
-#'@description This uses the `maptools::placeLabel()` function to generate labels for the given scene. Either
+#'@description This uses the `car::placeLabel()` function to generate labels for the given scene. Either
 #'use an `sf` object or manually specify the x/y coordinates and label. 
 #'
 #'@param labels A character vector of labels, or an `sf` object with `POINT` geometry and a column for labels.
-#'@param extent A `raster::Extent` object with the bounding box for the height map used to generate the original map.
+#'@param extent Either an object representing the spatial extent of the scene 
+#' (either from the `raster`, `terra`, `sf`, or `sp` packages), 
+#' a length-4 numeric vector specifying `c("xmin", "xmax","ymin","ymax")`, or the spatial object (from 
+#' the previously aforementioned packages) which will be automatically converted to an extent object. 
 #'@param x Default `NULL`. The x-coordinate, if `labels` is not an `sf` object.
 #'@param y Default `NULL`. The y-coordinate, if `labels` is not an `sf` object.
 #'@param heightmap Default `NULL`. The original height map. Pass this in to extract the dimensions of the resulting 
@@ -40,12 +43,12 @@
 #'@export
 #'@examples
 #'#Add the included `sf` object with roads to the montereybay dataset
-#'#Only run these examples if the `magick` package is installed.
-#'if ("magick" %in% rownames(utils::installed.packages())) {
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'#Create the water palette
 #'water_palette = colorRampPalette(c("darkblue", "dodgerblue", "lightblue"))(200)
 #'bathy_hs = height_shade(montereybay, texture = water_palette)
+#'#Set label font
+#'par(family = "Arial")
 #'
 #'#We're plotting the polygon data here for counties around Monterey Bay. We'll first
 #'#plot the county names at the polygon centroids.
@@ -60,7 +63,8 @@
 #'                                     extent= attr(montereybay,"extent"), heightmap = montereybay,
 #'                                     seed=1))  %>%
 #'  plot_map()
-#'
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#It's hard to read these values, so we'll add a white halo.
 #'bathy_hs %>% 
 #'  add_shadow(lamb_shade(montereybay,zscale=50),0.3) %>%
@@ -74,8 +78,8 @@
 #'                                     halo_color = "white", halo_expand = 3,
 #'                                     seed=1))  %>%
 #'  plot_map()
-#'
-#'
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Plot the actual town locations, using the manual plotting interface instead of the `sf` object
 #'montereybay %>%
 #'  height_shade() %>%
@@ -89,7 +93,8 @@
 #'                                     halo_color = "white", halo_expand = 3,
 #'                                     seed=1))  %>%
 #'  plot_map()
-#'
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Adding a softer blurred halo
 #'montereybay %>%
 #'  height_shade() %>%
@@ -103,7 +108,8 @@
 #'                                     halo_color = "white", halo_expand = 3, halo_blur=10,
 #'                                     seed=1))  %>%
 #'  plot_map()
-#'  
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Changing the seed changes the locations of the labels
 #'montereybay %>%
 #'  height_shade() %>%
@@ -118,7 +124,6 @@
 #'                                     seed=2))  %>%
 #'  plot_map()
 #'}
-#'}
 generate_label_overlay = function(labels, extent, x=NULL, y=NULL, 
                                   heightmap = NULL, width=NA, height=NA, text_size = 1,
                                   color = "black", font = 1, pch = 16, 
@@ -135,13 +140,13 @@ generate_label_overlay = function(labels, extent, x=NULL, y=NULL,
   if(is.na(point_color)) {
     point_color = color
   }
-  if(!("maptools" %in% rownames(utils::installed.packages()))) {
-    stop("{maptools} package required for generate_label_overlay()")
+  if(!(length(find.package("car", quiet = TRUE)) > 0)) {
+    stop("{car} package required for generate_label_overlay()")
   }
   if((inherits(labels,"sf") || inherits(labels,"sfc")) && 
      (is.null(x) && is.null(y)) &&
      !is.null(data_label_column)) {
-    if(!("sf" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("sf", quiet = TRUE)) > 0)) {
       stop("{sf} package required for {sf} object support")
     }
     geometry_list = sf::st_geometry(labels)
@@ -165,6 +170,7 @@ generate_label_overlay = function(labels, extent, x=NULL, y=NULL,
   stopifnot(!missing(extent))
   stopifnot(!missing(labels))
 
+  extent = get_extent(extent)
   if(is.na(height)) {
     height  = ncol(heightmap)
   }
@@ -174,30 +180,30 @@ generate_label_overlay = function(labels, extent, x=NULL, y=NULL,
   tempoverlay = tempfile(fileext = ".png")
   grDevices::png(filename = tempoverlay, width = width, height = height, units="px",bg = "transparent")
   graphics::par(mar = c(0,0,0,0))
-  graphics::plot(x=x+offset[1],y=y+offset[2], xlim = c(extent@xmin,extent@xmax),
-                 ylim =  c(extent@ymin,extent@ymax), asp=1, pch = pch,bty="n",axes=FALSE,
+  graphics::plot(x=x+offset[1],y=y+offset[2], xlim = c(extent["xmin"],extent["xmax"]),
+                 ylim =  c(extent["ymin"],extent["ymax"]), asp=1, pch = pch,bty="n",axes=FALSE,
                  xaxs = "i", yaxs = "i", cex = point_size, col = point_color)
-  maptools::pointLabel(x=x+offset[1],y=y+offset[2],labels=labels, cex = text_size,
-                 xlim = c(extent@xmin,extent@xmax), bty="n",
-                 ylim =  c(extent@ymin,extent@ymax), asp=1, font = font,
-                 xaxs = "i", yaxs = "i",  col = color)
+  car::pointLabel(x=x+offset[1],y=y+offset[2],labels=labels, cex = text_size,
+                  xlim = c(extent["xmin"],extent["xmax"]), bty="n",
+                  ylim = c(extent["ymin"],extent["ymax"]), asp=1, font = font,
+                  xaxs = "i", yaxs = "i",  col = color)
   grDevices::dev.off() #resets par
   overlay_temp = png::readPNG(tempoverlay)
   if(!is.na(halo_color)) {
-    if(!("rayimage" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("rayimage", quiet = TRUE)) > 0)) {
       stop("{rayimage} package required for `halo_color`")
     }
     set.seed(seed)
     tempoverlay = tempfile(fileext = ".png")
     grDevices::png(filename = tempoverlay, width = width, height = height, units="px",bg = "transparent")
     graphics::par(mar = c(0,0,0,0))
-    graphics::plot(x=x+halo_offset[1],y=y+halo_offset[2], xlim = c(extent@xmin,extent@xmax),
-                   ylim =  c(extent@ymin,extent@ymax), asp=1, pch = pch, bty="n",axes=FALSE,
+    graphics::plot(x=x+halo_offset[1],y=y+halo_offset[2], xlim = c(extent["xmin"],extent["xmax"]),
+                   ylim =  c(extent["ymin"],extent["ymax"]), asp=1, pch = pch, bty="n",axes=FALSE,
                    xaxs = "i", yaxs = "i", cex = point_size, col = halo_color)
-    maptools::pointLabel(x=x+halo_offset[1],y=y+halo_offset[2],labels=labels, cex = text_size, 
-                         xlim = c(extent@xmin,extent@xmax), bty="n", 
-                         ylim =  c(extent@ymin,extent@ymax), asp=1, font = font,
-                         xaxs = "i", yaxs = "i",  col = halo_color)
+    car::pointLabel(x=x+halo_offset[1],y=y+halo_offset[2],labels=labels, cex = text_size, 
+                    xlim = c(extent["xmin"],extent["xmax"]), bty="n", 
+                    ylim = c(extent["ymin"],extent["ymax"]), asp=1, font = font,
+                    xaxs = "i", yaxs = "i",  col = halo_color)
     grDevices::dev.off() #resets par
     overlay_temp_under = png::readPNG(tempoverlay)
     if(halo_expand != 0 || any(halo_offset != 0)) {
@@ -223,4 +229,4 @@ generate_label_overlay = function(labels, extent, x=NULL, y=NULL,
     return(add_overlay(overlay_temp_under, overlay_temp))
   }
   return(overlay_temp)
-}
\ No newline at end of file
+}
