@@ -6,14 +6,16 @@
 #'@param heightmap A two-dimensional matrix, where each entry in the matrix is the elevation at that point. All points are assumed to be evenly spaced.
 #'@param zscale Default `1`. The ratio between the x and y spacing (which are assumed to be equal) and the z axis. For example, if the elevation levels are in units
 #'of 1 meter and the grid values are separated by 10 meters, `zscale` would be 10. Adjust the zscale down to exaggerate elevation features.
-#'@param baseshape Default `rectangle`. Shape of the base. Options are c("rectangle","circle","hex").
+#'@param baseshape Default `rectangle`. Shape of the base. Options are `c("rectangle","circle","hex")`.
 #'@param solid Default `TRUE`. If `FALSE`, just the surface is rendered.
-#'@param soliddepth Default `auto`, which sets it to the lowest elevation in the matrix minus one unit (scaled by zscale). Depth of the solid base.
+#'@param soliddepth Default `auto`, which sets it to the lowest elevation in the matrix minus one unit (scaled by zscale). Depth of the solid base. If heightmap is uniform and set on `auto`, this is automatically set to a slightly lower level than the uniform elevation.
 #'@param solidcolor Default `grey20`. Base color.
 #'@param solidlinecolor Default `grey30`. Base edge line color.
 #'@param shadow Default `TRUE`. If `FALSE`, no shadow is rendered.
 #'@param shadowdepth Default `auto`, which sets it to `soliddepth - soliddepth/10`. Depth of the shadow layer.
-#'@param shadowcolor Default `grey50`. Color of the shadow.
+#'@param shadowcolor Default `auto`. Color of the shadow, automatically computed as `shadow_darkness`
+#'the luminance of the `background` color in the CIELab colorspace if not specified.
+#'@param shadow_darkness Default `0.5`. Darkness of the shadow, if `shadowcolor = "auto"`.
 #'@param shadowwidth Default `auto`, which sizes it to 1/10th the smallest dimension of `heightmap`. Width of the shadow in units of the matrix. 
 #'@param water Default `FALSE`. If `TRUE`, a water layer is rendered.
 #'@param waterdepth Default `0`. Water level.
@@ -23,6 +25,14 @@
 #'@param waterlinealpha Default `1`. Water line tranparency. 
 #'@param linewidth Default `2`. Width of the edge lines in the scene.
 #'@param lineantialias Default `FALSE`. Whether to anti-alias the lines in the scene.
+#'@param soil Default `FALSE`. Whether to draw the solid base with a textured soil layer.
+#'@param soil_freq Default `0.1`. Frequency of soil clumps. Higher frequency values give smaller soil clumps.
+#'@param soil_levels Default `16`. Fractal level of the soil.
+#'@param soil_color_light Default `"#b39474"`. Light tint of soil.
+#'@param soil_color_dark Default `"#8a623b"`. Dark tint of soil.
+#'@param soil_gradient Default `2`. Sharpness of the soil darkening gradient. `0` turns off the gradient entirely.
+#'@param soil_gradient_darken Default `4`. Amount to darken the `soil_color_dark` value for the deepest soil layers. Higher
+#'numbers increase the darkening effect.
 #'@param theta Default `45`. Rotation around z-axis.
 #'@param phi Default `45`. Azimuth angle.
 #'@param fov Default `0`--isometric. Field-of-view angle.
@@ -53,45 +63,62 @@
 #'to save a smaller 3D OBJ file to disk with `save_obj()`,
 #'@param verbose Default `TRUE`, if `interactive()`. Prints information about the mesh triangulation
 #'if `triangulate = TRUE`.
+#'@param plot_new Default `TRUE`, opens new window with each `plot_3d()` call. If `FALSE`, 
+#'the data will be plotted in the same window.
+#'@param close_previous Default `TRUE`. Closes any previously open `rgl` window. If `FALSE`, 
+#'old windows will be kept open.
+#'@param clear_previous Default `TRUE`. Clears the previously open `rgl` window if `plot_new = FALSE`.
 #'@param ... Additional arguments to pass to the `rgl::par3d` function.
+#'
 #'@import rgl
 #'@export
 #'@examples
-#'if(interactive()) {
 #'#Plotting a spherical texture map of the built-in `montereybay` dataset.
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade(texture="desert") %>%
 #'  plot_3d(montereybay,zscale=50)
-#'render_snapshot(clear = TRUE)
+#'render_snapshot()
 #'}
 #'
 #'#With a water layer  
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade(texture="imhof2") %>%
 #'  plot_3d(montereybay, zscale=50, water = TRUE, watercolor="imhof2", 
 #'          waterlinecolor="white", waterlinealpha=0.5)
-#'render_snapshot(clear = TRUE)
+#'render_snapshot()
+#'}
+#'
+#'#With a soil texture to the base  
+#'if(rayshader:::run_documentation()) {
+#'montereybay %>%
+#'  sphere_shade(texture="imhof3") %>%
+#'  plot_3d(montereybay, zscale=50, water = TRUE,  watercolor="imhof4", 
+#'          waterlinecolor="white", waterlinealpha=0.5, soil=TRUE)
+#'render_camera(theta=225, phi=7, zoom=0.5, fov=67)
+#'render_snapshot()
 #'}
 #'
 #'#We can also change the base by setting "baseshape" to "hex" or "circle"
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade(texture="imhof1") %>%
 #'  plot_3d(montereybay, zscale=50, water = TRUE, watercolor="imhof1", theta=-45, zoom=0.7,
 #'          waterlinecolor="white", waterlinealpha=0.5,baseshape="circle")
-#'render_snapshot(clear = TRUE)
+#'render_snapshot()
 #'}
 #'
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade(texture="imhof1") %>%
 #'  plot_3d(montereybay, zscale=50, water = TRUE, watercolor="imhof1", theta=-45, zoom=0.7,
 #'          waterlinecolor="white", waterlinealpha=0.5,baseshape="hex")
-#'render_snapshot(clear = TRUE)
+#'render_snapshot()
 #'}
 #'
+#'
+#'
 #'#Or we can carve out the region of interest ourselves, by setting those entries to NA
 #'#to the elevation map passed into `plot_3d`
 #'
@@ -101,24 +128,31 @@
 #'mb_water = montereybay
 #'mb_water[mb_water > -10] = NA
 #'
-#'\donttest{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade(texture="imhof1") %>%
 #'  plot_3d(mb_water, zscale=50, water = TRUE, watercolor="imhof1", theta=-45,
 #'          waterlinecolor="white", waterlinealpha=0.5)
-#'render_snapshot(clear = TRUE)
-#'}
+#'render_snapshot()
 #'}
 plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
                    solid = TRUE, soliddepth="auto", solidcolor="grey20",solidlinecolor="grey30",
-                   shadow = TRUE, shadowdepth = "auto", shadowcolor = "grey50", shadowwidth = "auto", 
+                   shadow = TRUE, shadowdepth = "auto", shadowcolor = "auto", shadow_darkness = 0.5,
+                   shadowwidth = "auto", 
                    water = FALSE, waterdepth = 0, watercolor="dodgerblue", wateralpha = 0.5, 
                    waterlinecolor=NULL, waterlinealpha = 1, 
-                   linewidth = 2, lineantialias = FALSE,
+                   linewidth = 2, lineantialias = FALSE, 
+                   soil = FALSE, soil_freq = 0.1, soil_levels = 16, 
+                   soil_color_light = "#b39474", soil_color_dark = "#8a623b", 
+                   soil_gradient = 2, soil_gradient_darken = 4,
                    theta=45, phi = 45, fov=0, zoom = 1, background="white", windowsize = 600,
                    precomputed_normals = NULL, asp = 1,
                    triangulate = FALSE, max_error = 0, max_tri = 0, verbose = FALSE,
+                   plot_new = TRUE, close_previous = TRUE, clear_previous = TRUE,
                    ...) {
+  if(!plot_new && clear_previous) {
+    rgl::clear3d()
+  }
   #setting default zscale if montereybay is used and tell user about zscale
   argnameschar = unlist(lapply(as.list(sys.call()),as.character))[-1]
   argnames = as.list(sys.call())
@@ -137,6 +171,9 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
       }
     }
   }
+  if(shadowcolor == "auto") {
+    shadowcolor = convert_color(darken_color(background, darken=shadow_darkness), as_hex = TRUE)
+  }
   #Set window size and position
   if(length(windowsize) == 1) {
     windowsize = c(0,0,windowsize,windowsize)
@@ -149,41 +186,8 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
   } else {
     stop(paste0("Don't know what to do with `windowsize` argument of length ",length(windowsize)))
   }
-  if(baseshape == "circle") {
-    radius = ifelse(nrow(heightmap) <= ncol(heightmap),nrow(heightmap)/2-1,ncol(heightmap)/2-1)
-    radmat = gen_circle_psf(radius+1)
-    if(min(dim(heightmap)) != min(dim(radmat))) {
-      radmat = radmat[2:nrow(radmat),2:ncol(radmat)]
-    }
-    if(max(dim(heightmap)) != max(dim(radmat))) {
-      difference = max(dim(heightmap)) - max(dim(radmat))
-      radtemp = matrix(0,nrow=nrow(heightmap),ncol=ncol(heightmap))
-      if(ncol(heightmap) != ncol(radmat)) {
-        radtemp[,(difference/2):(difference/2+ncol(radmat)-1)] = radmat
-      } else {
-        radtemp[(difference/2):(difference/2+nrow(radmat)-1),] = radmat
-      }
-      radmat = radtemp
-    }
-    heightmap[radmat == 0] = NA
-  } else if(baseshape == "hex") {
-    radius = ifelse(nrow(heightmap) <= ncol(heightmap),nrow(heightmap)/2-1,ncol(heightmap)/2-1)
-    radmat = gen_hex_psf(radius+1,rotation = 0)
-    if(min(dim(heightmap)) != min(dim(radmat))) {
-      radmat = radmat[2:nrow(radmat),2:ncol(radmat)]
-    }
-    if(max(dim(heightmap)) != max(dim(radmat))) {
-      difference = max(dim(heightmap)) - max(dim(radmat))
-      radtemp = matrix(0,nrow=nrow(heightmap),ncol=ncol(heightmap))
-      if(ncol(heightmap) != ncol(radmat)) {
-        radtemp[,(difference/2):(difference/2+ncol(radmat)-1)] = radmat
-      } else {
-        radtemp[(difference/2):(difference/2+nrow(radmat)-1),] = radmat
-      }
-      radmat = radtemp
-    }
-    heightmap[radmat == 0] = NA
-  }
+  heightmap = generate_base_shape(heightmap, baseshape)
+  
   if(any(hillshade > 1 | hillshade < 0, na.rm = TRUE)) {
     stop("Argument `hillshade` must not contain any entries less than 0 or more than 1")
   }
@@ -193,14 +197,55 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
   if(is.null(heightmap)) {
     stop("heightmap argument missing--need to input both hillshade and original elevation matrix")
   }
+  min_height = min(heightmap,na.rm=TRUE)
+  max_height = max(heightmap,na.rm=TRUE)
   if(soliddepth == "auto") {
-    soliddepth = min(heightmap,na.rm = TRUE)/zscale - (max(heightmap,na.rm = TRUE)/zscale-min(heightmap,na.rm = TRUE)/zscale)/5
+    if(min_height != max_height) {
+      soliddepth = min_height/zscale - (max_height/zscale-min_height/zscale)/5
+    } else {
+      max_dim = max(dim(heightmap))
+      soliddepth = min_height/zscale - max_dim/25
+    }
+  } else {
+    if(soliddepth > min_height) {
+      message(sprintf("`soliddepth` (set to %f) must be less than or equal to heightmap minimum value (%f). Setting to min(heightmap)",
+                      soliddepth, min_height))
+      soliddepth = min_height/zscale
+    } else {
+      soliddepth = soliddepth/zscale
+    }
+  }
+  if(solid) {
+    min_height_shadow = min(c(min_height, soliddepth*zscale))
+  } else {
+    min_height_shadow = min_height
   }
   if(shadowdepth == "auto") {
-    shadowdepth = soliddepth - (max(heightmap,na.rm = TRUE)/zscale-min(heightmap,na.rm = TRUE)/zscale)/5
+    if(min_height_shadow != max_height) {
+      if(solid) {
+        shadowdepth = soliddepth - (max_height/zscale-min_height_shadow/zscale)/5
+      } else {
+        shadowdepth = min_height_shadow/zscale - (max_height/zscale-min_height_shadow/zscale)/5
+      }
+    } else {
+      if(solid) {
+        max_dim = max(dim(heightmap))
+        shadowdepth = soliddepth - max_dim/25
+      } else {
+        max_dim = max(dim(heightmap))
+        shadowdepth = min_height - max_dim/25
+      }
+    }
+  } else {
+    if(shadowdepth > min_height) {
+      message(sprintf("`shadowdepth` (set to %f) is greater to heightmap minimum value (%f). Shadow will appear to be intersecting 3D model.",
+                      shadowdepth, min_height))
+    } else {
+      shadowdepth = shadowdepth/zscale
+    }
   }
   if(shadowwidth == "auto") {
-    shadowwidth = floor(min(dim(heightmap))/10)
+    shadowwidth = max(floor(min(dim(heightmap))/10),5)
   }
   if(water) {
     if (watercolor == "imhof1") {
@@ -236,7 +281,7 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
       waterlinecolor = "#ffd1fb"
     }
   }
-  tempmap = tempfile()
+  tempmap = tempfile(fileext=".png")
   save_png(hillshade,tempmap)
   precomputed = FALSE
   if(is.list(precomputed_normals)) {
@@ -249,21 +294,36 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
     }
     triangulate = FALSE
   }
+  
+  if(close_previous && rgl::cur3d() != 0) {
+    rgl::close3d()
+  }
+  if(plot_new || rgl::cur3d() == 0) {
+    rgl::open3d(windowRect = windowsize, 
+                mouseMode = c("none", "polar", "fov", "zoom", "pull"))
+  }
+  rgl::view3d(zoom = zoom, phi = phi, theta = theta, fov = fov)
+  attributes(heightmap) = attributes(heightmap)["dim"]
   if(!triangulate) {
     if(!precomputed) {
-      normals = calculate_normal(heightmap,zscale=zscale)
+      normals = calculate_normal(heightmap, zscale = zscale)
     }
-    normalsx = (t(normals$x[c(-1,-nrow(normals$x)),c(-1,-ncol(normals$x))]))
-    normalsy = (t(normals$z[c(-1,-nrow(normals$z)),c(-1,-ncol(normals$z))]))
-    normalsz = (t(normals$y[c(-1,-nrow(normals$y)),c(-1,-ncol(normals$y))]))
-    rgl.surface(x=1:nrow(heightmap)-nrow(heightmap)/2,z=(1:ncol(heightmap)-ncol(heightmap)/2),
-                y=heightmap/zscale,
-                normal_x = normalsz, normal_y = normalsy, normal_z = -normalsx,
-                texture=paste0(tempmap,".png"),lit=FALSE,ambient = "#000001")
+    dim(heightmap) = unname(dim(heightmap))
+    normalsx = (t(normals$x[c(-1, -nrow(normals$x)), c(-1,-ncol(normals$x))]))
+    normalsy = (t(normals$z[c(-1, -nrow(normals$z)), c(-1,-ncol(normals$z))]))
+    normalsz = (t(normals$y[c(-1, -nrow(normals$y)), c(-1,-ncol(normals$y))]))
+    ray_surface = generate_surface(heightmap, zscale = zscale)
+    rgl::triangles3d(x = ray_surface$verts, 
+                     indices = ray_surface$inds, 
+                     texcoords = ray_surface$texcoords, 
+                     normals = matrix(c(normalsz,normalsy,-normalsx), ncol = 3L),
+                     texture = tempmap, color="white", lit = FALSE, tag = "surface_tris",
+                     back = "culled")
   } else {
     tris = terrainmeshr::triangulate_matrix(heightmap, maxError = max_error, 
-                                            maxTriangles = max_tri, start_index = 0, 
+                                            maxTriangles = max_tri, start_index = 0L, 
                                             verbose = verbose)
+    index_vals = seq_len(nrow(tris))
     # if(!precomputed) {
     #   normals = calculate_normal(heightmap,zscale=zscale)
     # }
@@ -277,24 +337,26 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
     cn = tris[,3]+1
     # normal_comp = matrix(c(normalsz[rn + nr*(cn-1)],normalsy[rn + nr*(cn-1)],-normalsx[rn + nr*(cn-1)]),ncol=3)
     texcoords = tris[,c(1,3)]
-    texcoords[,1] = texcoords[,1]/nrow(heightmap)
-    texcoords[,2] = texcoords[,2]/ncol(heightmap)
+    texcoords[,1] = texcoords[,1]/(nrow(heightmap)-1)
+    texcoords[,2] = texcoords[,2]/(ncol(heightmap)-1)
     tris[,1] = tris[,1] - nrow(heightmap)/2 +1
     tris[,3] = tris[,3] - ncol(heightmap)/2
     tris[,3] = -tris[,3]
-    rgl.triangles(tris, texcoords = texcoords, #normals = normal_comp,
-                  texture=paste0(tempmap,".png"),lit=FALSE,ambient = "#000017")
+    rgl::triangles3d(tris, texcoords = texcoords, 
+                     indices = index_vals,
+                     #normals = normal_comp,
+                     texture=tempmap,lit=FALSE,color="white",tag = "surface_tris")
   }
-  bg3d(color = background,texture=NULL)
-  rgl.viewpoint(zoom=zoom,phi=phi,theta=theta,fov=fov)
-  par3d(windowRect = windowsize,...)
+  rgl::bg3d(color = background,texture=NULL)
+  # rgl::par3d(windowRect = windowsize, mouseMode = c("none", "polar", "fov", "zoom", "pull"), ...)
   if(solid && !triangulate) {
-    make_base(heightmap,basedepth=soliddepth,basecolor=solidcolor,zscale=zscale)
+    make_base(heightmap,basedepth=soliddepth,basecolor=solidcolor,zscale=zscale, 
+              soil = soil, soil_freq = soil_freq, soil_levels = soil_levels, soil_color1=soil_color_light,
+              soil_color2=soil_color_dark, soil_gradient = soil_gradient, gradient_darken = soil_gradient_darken)
   } else if(solid && triangulate) {
     make_base_triangulated(tris,basedepth=soliddepth,basecolor=solidcolor)
   }
   if(!is.null(solidlinecolor) && solid) {
-    rgl::rgl.material(color=solidlinecolor, lit=FALSE)
     make_lines(heightmap,basedepth=soliddepth,linecolor=solidlinecolor,zscale=zscale,linewidth = linewidth)
   }
   if(shadow) {
@@ -305,11 +367,9 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
   }
   if(!is.null(waterlinecolor) && water) {
     if(all(!is.na(heightmap))) {
-      rgl::rgl.material(color=waterlinecolor,lit=FALSE)
       make_lines(fliplr(heightmap),basedepth=waterdepth,linecolor=waterlinecolor,
                  zscale=zscale,linewidth = linewidth,alpha=waterlinealpha,solid=FALSE)
     }
-    rgl::rgl.material(color=waterlinecolor,lit=FALSE)
     make_waterlines(heightmap,waterdepth=waterdepth,linecolor=waterlinecolor,
                     zscale=zscale,alpha=waterlinealpha,linewidth=linewidth,antialias=lineantialias)
   }
@@ -318,4 +378,4 @@ plot_3d = function(hillshade, heightmap, zscale=1, baseshape="rectangle",
     height_scale = height_range[2]-height_range[1]
     rgl::aspect3d(x = dim(heightmap)[1]/height_scale, y = 1, z = dim(heightmap)[2]*asp/height_scale)
   }
-}
\ No newline at end of file
+}
