@@ -7,7 +7,10 @@
 #'@param lat A latitude for the text. Must provide an `raster::extent` object to argument `extent` for the map.
 #'@param long A latitude for the text. Must provide an `raster::extent` object to argument `extent` for the map.
 #'@param altitude Default `NULL`. Elevation of the label, in units of the elevation matrix (scaled by zscale). If none is passed, this will default to 10 percent above the maximum altitude in the heightmap.
-#'@param extent Default `NULL`. A `raster::Extent` object with the bounding box of the displayed 3D scene.
+#'@param extent Either an object representing the spatial extent of the scene 
+#' (either from the `raster`, `terra`, `sf`, or `sp` packages), 
+#' a length-4 numeric vector specifying `c("xmin", "xmax","ymin","ymax")`, or the spatial object (from 
+#' the previously aforementioned packages) which will be automatically converted to an extent object. 
 #'@param x Default `NULL`. Directly specify the `x` index in the matrix to place the label.
 #'@param y Default `NULL`. Directly specify the `y` index in the matrix to place the label.
 #'@param z Default `NULL`. Elevation of the label, in units of the elevation matrix (scaled by zscale).
@@ -31,8 +34,7 @@
 #'@param textcolor Default `black`. Color of the text.
 #'@export
 #'@examples
-#'if(interactive()) {
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade() %>%
 #'  plot_3d(montereybay,zscale=50,water=TRUE, watercolor="#233aa1")
@@ -41,7 +43,7 @@
 #'
 #'santa_cruz = c(36.962957, -122.021033) 
 #'#We want to add a label to Santa Cruz, so we use the x and y matrix coordinate (x=220 and y=330)
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_label(montereybay,lat = santa_cruz[1], long = santa_cruz[2],
 #'             extent = attr(montereybay, "extent"),
 #'             altitude=12000, zscale=50, text = "Santa Cruz")
@@ -52,7 +54,7 @@
 #'#We can also change the linetype to dashed by setting `dashed = TRUE` (additional options allow
 #'#the user to control the dash length). You can clear the existing lines by setting 
 #'#`clear_previous = TRUE`.
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_label(montereybay, lat = monterey[1], long = monterey[2], altitude = 10000, 
 #'             extent = attr(montereybay, "extent"),
 #'             zscale = 50, text = "Monterey", textcolor = "white", linecolor="darkred",
@@ -63,7 +65,7 @@
 #'canyon = c(36.621049, -122.333912)
 #'#By default, z specifies the altitude above that point on the elevation matrix. We can also specify 
 #'#an absolute height by setting `relativez=FALSE`.
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_label(montereybay,lat=canyon[1], long = canyon[2], altitude = 2000,
 #'             extent = attr(montereybay,"extent"),
 #'             zscale=50,text = "Monterey Canyon", relativez=FALSE)
@@ -73,7 +75,7 @@
 #'#We can also render labels in high quality with `render_highquality()`, specifying a custom
 #'#line radius. By default, the labels point towards the camera, but you can fix their angle with
 #'#argument `text_angle`.
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_camera(theta=35, phi = 35, zoom = 0.80, fov=60)
 #'render_label(montereybay, lat = monterey[1], long = monterey[2], altitude = 10000, 
 #'             extent = attr(montereybay, "extent"),
@@ -84,19 +86,20 @@
 #'             extent = attr(montereybay,"extent"), textcolor = "white", linecolor="white",
 #'             text = "Monterey Canyon", relativez=FALSE)
 #'             
-#'render_highquality(samples=200,text_size = 24, line_radius = 2, text_offset = c(0,20,0),
-#'                   lightdirection=180, clamp_value=10)
-#'                   
+#'render_highquality(samples = 128,text_size = 24, line_radius = 2, text_offset = c(0, 20, 0),
+#'                   lightdirection = 180, clamp_value = 10, min_variance = 0,
+#'                   sample_method = "sobol_blue")
+#'}
+#'if(rayshader:::run_documentation()) {
 #'#Fixed text angle
-#'render_highquality(samples=200,text_size = 24, line_radius = 2, text_offset = c(0,20,0),
-#'                   lightdirection=180, text_angle=0, clamp_value=10)
+#'render_highquality(samples = 128,text_size = 24, line_radius = 2, text_offset = c(0, 20, 0),
+#'                   lightdirection = 180, text_angle = 0, clamp_value=10, min_variance = 0,
+#'                   sample_method = "sobol_blue")
 #'}
 #'#We can remove all existing labels by calling `render_label(clear_previous = TRUE)`
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_label(clear_previous = TRUE) 
 #'render_snapshot()
-#'rgl::rgl.close()
-#'}
 #'}
 render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL, 
                         x=NULL, y=NULL, z=NULL, zscale=1, 
@@ -107,13 +110,9 @@ render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL,
                         linecolor = "black", textcolor = "black") {
   exit_early = FALSE
   if(clear_previous) {
-    ray_text_ids = get_ids_with_labels(c("textline", "raytext"))
-    if(nrow(ray_text_ids) > 0 || missing(text)) {
-      remove_ids = ray_text_ids$id
-      rgl::pop3d(id = remove_ids)
-      if(missing(text)) {
-        exit_early = TRUE
-      }
+    rgl::pop3d(tag = c("textline", "raytext"))
+    if(missing(text)) {
+      exit_early = TRUE
     }
   }
   if(!exit_early) {
@@ -121,17 +120,17 @@ render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL,
       z = altitude
     }
     if(is.null(z)) {
-      z = max(heightmap)*1.1
+      z = max(heightmap,na.rm=TRUE)*1.1
     }
     if(is.null(extent) && (!missing(lat) || !missing(long)) && (!is.null(x) && !is.null(y))) {
       stop("extent required when using lat/long instead of x/y")
     }
     if(!is.null(extent)) {
-      e = extent
-      x = (long-e@xmin)/(e@xmax - e@xmin) * nrow(heightmap)
-      y = ncol(heightmap) - (lat-e@ymin)/(e@ymax - e@ymin) * ncol(heightmap)
+      e = get_extent(extent)
+      x = (long-e["xmin"])/(e["xmax"] - e["xmin"]) * nrow(heightmap)
+      y = ncol(heightmap) - (lat-e["ymin"])/(e["ymax"] - e["ymin"]) * ncol(heightmap)
     }
-    if(rgl::rgl.cur() == 0) {
+    if(rgl::cur3d() == 0) {
       stop("No rgl window currently open.")
     }
     if(.Platform$OS.type == "unix") {
@@ -141,10 +140,24 @@ render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL,
     }
     fontlist = list("standard"=1,"bold"=2,"italic"=3,"bolditalic"=4)
     fonttype = fontlist[[fonttype]]
+    in_bounds = TRUE
+    if(x > nrow(heightmap) || x < 1 || y < 1 || y > ncol(heightmap) || is.na(heightmap[x,y])) {
+      in_bounds = FALSE
+      shadow_id = get_ids_with_labels("shadow")$id
+      if(length(shadow_id) > 0) {
+        shadow_vertices = rgl::rgl.attrib(shadow_id, "vertices")
+        startline = min(shadow_vertices[,2],na.rm=TRUE)
+      } else {
+        offset = startline
+      }
+    } 
+    
     z=z/zscale
     offset = offset/zscale
-    startline = heightmap[x,y]/zscale
-    if(relativez)  {
+    if(in_bounds) {
+      startline = heightmap[x,y]/zscale
+    } 
+    if(relativez && in_bounds)  {
       z = z + startline
     }
     if(dashlength == "auto") {
@@ -170,10 +183,9 @@ render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL,
       linelist[[1]] = matrix(c(x, x, z+offset, startline + offset, y,y),2,3)
     }
     for(i in 1:length(linelist)) {
-      rgl::rgl.material(color=linecolor)
       rgl::lines3d(linelist[[i]], color = linecolor, 
                    lwd = linewidth, lit = FALSE, line_antialias = antialias,
-                   depth_test = "less", alpha = alpha, ambient = "#000008")
+                   depth_test = "less", alpha = alpha, tag = "textline")
     }
     if(freetype) {
       seriflist = c("fonts/FreeSerif.ttf", "fonts/FreeSerifBold.ttf", 
@@ -281,7 +293,7 @@ render_label = function(heightmap, text, lat, long, altitude=NULL, extent=NULL,
     text3d(x, z+offset, y, 
            text,color=textcolor,adj=adjustvec,useFreeType=freetype,
            alpha=textalpha,family=family,font=fonttype,cex=textsize,
-           depth_test="less", ambient = "#000009", lit=FALSE)
+           depth_test="less", tag = "raytext", lit=FALSE)
     par3d(ignoreExtent = ignoreex)
   }
 }
