@@ -1,10 +1,11 @@
 #'@title Render Movie
 #'
-#'@description Renders a movie using the \pkg{av} package. Moves the camera around a 3D visualization 
+#'@description Renders a movie using the \pkg{av} or \pkg{gifski} packages. Moves the camera around a 3D visualization 
 #'using either a standard orbit, or accepts vectors listing user-defined values for each camera parameter. If the latter,
 #'the values must be equal in length to `frames` (or of length `1`, in which the value will be fixed).
 #'
-#'@param filename Filename. If not appended with `.mp4`, it will be appended automatically.
+#'@param filename Filename. If not appended with `.mp4`, it will be appended automatically. If the file extension is `gif`, 
+#'the \pkg{gifski} package will be used to generate the animation.
 #'@param type Default `orbit`, which orbits the 3D object at the user-set camera settings `phi`, `zoom`, and `fov`. 
 #'Other options are `oscillate` (sine wave around `theta` value, covering 90 degrees), or `custom` (which uses the values from the 
 #'`theta`, `phi`, `zoom`, and `fov` vectors passed in by the user).
@@ -14,6 +15,10 @@
 #'@param theta Default to current view. Theta values, in degrees. 
 #'@param zoom Defaults to the current view. Zoom value, between `0` and `1`. 
 #'@param fov Defaults to the current view. Field of view values, in degrees.
+#'@param width Default `NULL`, uses the window size by default. Width of the movie. Note that the frames will still
+#'be captured at the resolution (and aspect ratio) of the rgl window.
+#'@param height Default `NULL`, uses the window size by default. Height of the movie. Note that the frames will still
+#'be captured at the resolution (and aspect ratio) of the rgl window.
 #'@param title_text Default `NULL`. Text. Adds a title to the movie, using magick::image_annotate. 
 #'@param title_offset Default `c(20,20)`. Distance from the top-left (default, `gravity` direction in 
 #'image_annotate) corner to offset the title.
@@ -30,6 +35,8 @@
 #'@param vignette Default `FALSE`. If `TRUE` or numeric, a camera vignetting effect will be added to the image.
 #'`1` is the darkest vignetting, while `0` is no vignetting. If vignette is a length-2 vector, the second entry will
 #'control the blurriness of the vignette effect.
+#'@param vignette_color Default `"black"`. Color of the vignette.
+#'@param vignette_radius Default `1.3`. Radius of the vignette, as a porportion of the image dimensions.
 #'@param audio Default `NULL`. Optional file with audio to add to the video.
 #'@param progbar Default `TRUE` if interactive, `FALSE` otherwise. If `FALSE`, turns off progress bar. 
 #'Will display a progress bar when adding an overlay or title.
@@ -53,10 +60,6 @@
 #'#You can change to an oscillating orbit. The magnification is increased and azimuth angle set to 30.
 #'#A title has also been added using the title_text argument.
 #'\donttest{
-#'montereybay %>%
-#'  sphere_shade(texture="imhof1") %>%
-#'  plot_3d(montereybay, zscale=50, water = TRUE, watercolor="imhof1", 
-#'          waterlinecolor="white", waterlinealpha=0.5)
 #'#Un-comment the following to run:
 #'#render_movie(filename = filename_movie, type = "oscillate", 
 #'#             frames = 60,  phi = 30, zoom = 0.8, theta = -90,
@@ -73,32 +76,43 @@
 #'zoomvec = 0.45 + 0.2 * 1/(1 + exp(seq(-5, 20, length.out = 180)))
 #'zoomvecfull = c(zoomvec, rev(zoomvec))
 #'\donttest{
-#'montereybay %>%
-#'  sphere_shade(texture="imhof1") %>%
-#'  plot_3d(montereybay, zscale=50, water = TRUE, watercolor="imhof1", 
-#'          waterlinecolor="white", waterlinealpha=0.5)
 #'#Un-comment the following to run
 #'#render_movie(filename = filename_movie, type = "custom", 
 #'#             frames = 360,  phi = phivecfull, zoom = zoomvecfull, theta = thetavec)
-#'rgl::rgl.close()
 #'}
 #'}
 render_movie = function(filename, type = "orbit", frames = 360, fps = 30, 
                         phi = 30, theta = 0, zoom = NULL, fov = NULL, 
+                        width = NULL, height = NULL,
                         title_text = NULL, title_offset = c(20,20), 
                         title_color = "black", title_size = 30, title_font = "sans",
                         title_bar_color = NULL, title_bar_alpha = 0.5,
-                        image_overlay = NULL, vignette = FALSE, title_position = "northwest",
+                        image_overlay = NULL, 
+                        vignette = FALSE, vignette_color = "black", vignette_radius = 1.3,
+                        title_position = "northwest",
                         audio=NULL, progbar = interactive(), ...) {
-  if(rgl::rgl.cur() == 0) {
+  if(rgl::cur3d() == 0) {
     stop("No rgl window currently open.")
   }
-  if(!("av" %in% rownames(utils::installed.packages()))) {
-    stop("`av` package required for render_movie()")
-  }
   if(is.null(filename)) {
     stop("render_movie requires a filename")
   }
+  movie_type = tools::file_ext(filename)
+  use_av = TRUE
+  if(movie_type %in% c("mp4","mkv", "mov","flv","")) {
+    if(movie_type == "") {
+      filename = paste0(filename,".mp4")
+    }
+    if(!(length(find.package("av", quiet = TRUE)) > 0)) {
+      stop("`av` package required for render_movie()")
+    }
+  } else if (movie_type == "gif") {
+    if(!(length(find.package("gifski", quiet = TRUE)) > 0)) {
+      stop("`gifski` package required for render_movie() gifs")
+    }
+    use_av = FALSE
+  }
+
   if(!is.null(title_text)) {
     has_title = TRUE
   } else {
@@ -119,9 +133,6 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
   } else {
     has_overlay = FALSE
   }
-  if(substring(filename, nchar(filename)-3,nchar(filename)) != ".mp4") {
-    filename = paste0(filename,".mp4")
-  }
   windowsize = rgl::par3d()$viewport
   if(is.null(fov)) {
     fov = rgl::par3d()$FOV
@@ -148,13 +159,13 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
     theta_vector = seq(0,360,length.out = frames+1)[-(frames+1)]
     for(i in seq_len(frames)) {
       render_camera(theta = theta_vector[i], phi = phi, zoom = zoom, fov = fov)
-      rgl::snapshot3d(filename = png_files[i], top = FALSE)
+      rgl::snapshot3d(filename = png_files[i], top = FALSE, webshot = FALSE)
     }
   } else if (type == "oscillate") {
     theta_vector = theta + 45 * sin(seq(0,360,length.out = frames+1)[-(frames+1)]*pi/180)
     for(i in seq_len(frames)) {
       render_camera(theta = theta_vector[i], phi = phi, zoom = zoom, fov = fov)
-      rgl::snapshot3d(filename = png_files[i], top = FALSE)
+      rgl::snapshot3d(filename = png_files[i], top = FALSE, webshot = FALSE)
     }
   } else if (type == "custom") {
     if(length(theta) == 1) theta = rep(theta, frames)
@@ -166,18 +177,24 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
     }
     for(i in seq_len(frames)) {
       render_camera(theta = theta[i], phi = phi[i], zoom = zoom[i], fov = fov[i])
-      rgl::snapshot3d(filename = png_files[i], top = FALSE)
+      rgl::snapshot3d(filename = png_files[i], top = FALSE, webshot = FALSE)
     }
   } else {
     stop("Unknown type: ", type)
   }
   temp = png::readPNG(png_files[1])
   dimensions = dim(temp)
+  if(!is.null(width)) {
+    dimensions[1] = width
+  }
+  if(!is.null(height)) {
+    dimensions[2] = height
+  }
   if(dimensions[1] %% 2 != 0) {
     dimensions[1] = dimensions[1] - 1
   }
   if(has_overlay) {
-    if(!("magick" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("magick", quiet = TRUE)) > 0)) {
       stop("`magick` package required for adding overlay")
     }
     if(progbar) {
@@ -194,7 +211,7 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
     }
   }
   if(vignette || is.numeric(vignette)) {
-    if(!("magick" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("magick", quiet = TRUE)) > 0)) {
       stop("`magick` package required for adding overlay")
     }
     if(progbar) {
@@ -206,11 +223,13 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
       if(progbar) {
         pb$tick()
       }
-      rayimage::add_vignette(png_files[i], vignette = vignette, filename = png_files[i])
+      rayimage::add_vignette(png_files[i], vignette = vignette, filename = png_files[i], 
+                             color = vignette_color,
+                             radius = vignette_radius)
     }
   }
   if(has_title) {
-    if(!("magick" %in% rownames(utils::installed.packages()))) {
+    if(!(length(find.package("magick", quiet = TRUE)) > 0)) {
       stop("`magick` package required for adding title")
     }
     if(progbar) {
@@ -229,6 +248,11 @@ render_movie = function(filename, type = "orbit", frames = 360, fps = 30,
                           title_size = title_size, title_font = title_font)
     }
   }
-  av::av_encode_video(png_files, output = filename, framerate = fps, 
-                      vfilter = paste0("scale=",dimensions[1],":-2"), audio=audio)
+  if(use_av) {
+    av::av_encode_video(png_files, output = filename, framerate = fps, 
+                        vfilter = paste0("scale=",dimensions[2],":-2"), audio=audio)
+  } else {
+    gifski::gifski(png_files=png_files, gif_file = filename, delay = 1/fps, 
+                   width = dimensions[1],height= dimensions[2])
+  }
 }
