@@ -14,8 +14,7 @@
 #'@param remove_water Default `TRUE`. If `TRUE`, will remove existing water layer and replace it with new layer.
 #'@export
 #'@examples
-#'if(interactive()) {
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'montereybay %>%
 #'  sphere_shade() %>%
 #'  plot_3d(montereybay,zscale=50)
@@ -23,45 +22,39 @@
 #'}
 #'  
 #'#We want to add a layer of water after the initial render.
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_water(montereybay,zscale=50)
 #'render_snapshot()
 #'}
 #'
 #'#Call it again to change the water depth
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_water(montereybay,zscale=50,waterdepth=-1000)
 #'render_snapshot()
 #'}
 #'
 #'#Add waterlines
-#'\dontrun{
+#'if(rayshader:::run_documentation()) {
 #'render_camera(theta=-45)
 #'render_water(montereybay,zscale=50,waterlinecolor="white")
-#'render_snapshot(clear = TRUE)
-#'rgl::rgl.close()
-#'}
+#'render_snapshot()
 #'}
 render_water = function(heightmap, waterdepth=0, watercolor="lightblue",
                         zscale=1, wateralpha=0.5, waterlinecolor=NULL, waterlinealpha = 1, 
                         linewidth = 2, remove_water = TRUE) {
-  if(rgl::rgl.cur() == 0) {
+  if(rgl::cur3d() == 0) {
     stop("No rgl window currently open.")
   }
   if(remove_water) {
-    idlist = get_ids_with_labels()
-    remove_ids = idlist$id[idlist$raytype %in% c("waterlines", "water")]
-    rgl::pop3d(id=remove_ids)
+    rgl::pop3d(tag=c("waterlines", "water"))
   }
   make_water(heightmap/zscale,waterheight=waterdepth/zscale,wateralpha=wateralpha,watercolor=watercolor)
   if(!is.null(waterlinecolor)) {
     if(all(!is.na(heightmap))) {
-      rgl::rgl.material(color=waterlinecolor,lit=FALSE)
       make_lines(fliplr(heightmap),basedepth=waterdepth/zscale,linecolor=waterlinecolor,
                  zscale=zscale,linewidth = linewidth, alpha=waterlinealpha, solid=FALSE)
     }
-    rgl::rgl.material(color=waterlinecolor,lit=FALSE)
     make_waterlines(heightmap,waterdepth=waterdepth/zscale,linecolor=waterlinecolor,
                     zscale=zscale,alpha=waterlinealpha,linewidth=linewidth)
   }
-}
\ No newline at end of file
+}
